:- module(util,[count/2,oremove/3,distribution/4,dist_print/2,string_conc/2,string_conc_sep/3,osubset/3,osingle/3]).

:- use_module(library(lists)).
:- dynamic(count_n/1).
:- dynamic(dist_n/2).


% count the number of answers
count(X, _) :-
	retractall(count_n(_)),
	assert(count_n(0)),
	X,
	retract(count_n(M)),
	M1 is M+1,
	assert(count_n(M1)),
	fail.
	
count(_, N) :-
	count_n(N).


% distribution of the number of answers
distribution(X, M, Step, _) :-
	retractall(dist_n(_, _)),
	X,
	NStep is truncate(M/Step),
	dist_add(NStep),
	fail.
distribution(_, _, Step, R) :-
	findall( (N1, V), (dist_n(N, V), N1 is N*Step), R1),
	qsortdist(R1,R).

dist_print(File, R) :-
	open(File, write, S),
	dist_sum(R, Sum),
	dist_print0(S, Sum, R),
	close(S).

dist_sum([], 0).
dist_sum([(_,V)|R], S) :-
	dist_sum(R, S1),
	S is S1 + V.
	
dist_print0(_, _, []).
dist_print0(S, Sum, [(N,V)|R]) :-
	write(S, N),
	write(S, '\t'),
	write(S, V),
	write(S, '\t'),
	write(S, Sum),
	write(S, '\n'),
	Sum1 is Sum - V,
	dist_print0(S, Sum1, R).
	
dist_add(N) :-
	dist_n(N, V),!,
	V1 is V + 1,
	retract(dist_n(N, V)),
	assert(dist_n(N, V1)).

dist_add(N) :-
	assert(dist_n(N, 1)).

% oremove(A, B, C) for A, B sorted and unique
oremove(A, [], A).
oremove([X|Xs], [Y|Ys], [X|Zs]) :-
	X < Y,
	oremove(Xs, [Y|Ys], Zs).
oremove([X|Xs], [X|Ys], Zs) :-
	oremove(Xs, Ys, Zs).


qsortdist([], []).
qsortdist([(X,V)|Xs], Rs) :-
	split(X, Xs, Ls, Gs),
	qsortdist(Ls, LSorted),
	qsortdist(Gs, GSorted),
	append(LSorted, [(X,V)|GSorted], Rs).

% utility predicates for splitting, concatenating, etc.
split(_, [], [], []).
split(X, [(Y,V)|Xs], Ls, [(Y,V)|Gs]) :-
	X =< Y,
	split(X, Xs, Ls, Gs).
split(X, [(Y,V)|Xs], [(Y,V)|Ls], Gs) :-
	X > Y,
	split(X, Xs, Ls, Gs).

string_conc(Ls, SOut) :-
	string_conc0(Ls, '', SOut).
	
string_conc0([], S, S).
string_conc0([S1|Ss], SIn, SOut) :-
	string_concat(SIn, S1, SIn1),
	string_conc0(Ss, SIn1, SOut).

string_conc_sep([], _, '').
string_conc_sep([L|Ls], Sep, SOut) :-
	string_conc_sep0([L|Ls], '', Sep, SOut).
	
string_conc_sep0([S1], SIn, _, SOut) :-
	string_concat(SIn, S1, SOut).
string_conc_sep0([S1,S2|Ss], SIn, Sep, SOut) :-
	string_concat(SIn, S1, SIn1),
	string_concat(SIn1, Sep, SIn2),
	string_conc_sep0([S2|Ss], SIn2, Sep, SOut).

osubset([], [], []).
osubset([X|Xs], [X|Ys], Zs) :-
	osubset(Xs, Ys, Zs).
osubset([X|Xs], Ys, [X|Zs]) :-
	osubset(Xs, Ys, Zs).


osingle([X|Xs], [X], Xs).
osingle([X|Xs], Ys, [X|Zs]) :-
	osingle(Xs, Ys, Zs).


